
package de.bsvrz.ste.rdstmcvew;
/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
 
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.NumberArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Klasse zum Versenden von Daten unter dem Aspekt "Senden" der
 * Attributgruppe "RDSMeldung". 
 * Sie implementiert das ClientSenderInterface welches die Methoden
 * zum Versenden der Daten bereitstellt.
 * 
 * @author Dambach Werke GmbH
 * @version $Revision: 1.2 $ / $Date: 2009/02/20 15:12:04 $ / ($Author: Sans $)
 * 
 */
public class Meldung implements ClientSenderInterface
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Ist auf true gesetzt, solange auf Sendesteuerung gewartet wird
   */
  private boolean _warten = false;
  
  /**
   * Rolle, mit der als Sender angemeldet wird
   */
  public static final SenderRole QUELLE = SenderRole.source();  // Quelle
  
  public static final SenderRole SENDER = SenderRole.sender();  // Sender
  
  /**
   * Zeitin ms, die max. auf Sendesteuerung gewartet wird
   */
  public static final long SENDE_TIMEOUT = 5000;
  
  /**
   * Ist diese Instanz zum DAV-Senden angemeldet?
   */
  private boolean _angemeldet = false;
  
  /**
   * @return ist diese Instanz zum DAV-Senden angemeldet?
   */
  public boolean isAngemeldet()
  {
    return _angemeldet;
  }
  
  /**
   * Datenbeschreibung "generiert"
   */
  protected final DataDescription _datenBeschreibung;
  
  
  ClientDavInterface _connection;
  
  /**
   * @return die Datenverteiler-Verbindung
   */
  public ClientDavInterface getConnection()
  {
    return _connection;
  }
  
  private SystemObject _davObjekt;
  
  /**
   * @return das Dav-Systemobject, fr das das Senden von Daten erfolgt
   */
  public SystemObject getDavObjekt()
  {
    return _davObjekt;
  }
   
  /**
   * Konstruktor der Klasse
   * 
   * @param meldung Objekt vom Typ RDSVewMeldung, 
   * fr das das Senden von Daten erfolgt
   */
  public Meldung(ClientDavInterface connection, SystemObject davObjekt, String aspekt)
  {
    _connection = connection;
    _davObjekt = davObjekt;
    
    String text = null;
    
    AttributeGroup atg = getConnection().getDataModel().getAttributeGroup("atg.rdsMeldung");
    
    if(aspekt.equalsIgnoreCase("generiert"))
      text = "asp.rdsGeneriert";
      
    if(aspekt.equalsIgnoreCase("bearbeitet"))
      text = "asp.rdsBearbeitet";
 
    if(aspekt.equalsIgnoreCase("versendet"))
      text = "asp.rdsVersendet";
     
    Aspect asp = getConnection().getDataModel().getAspect(text);
    
    _datenBeschreibung = new DataDescription(atg, asp);
    
    try
    {
      if(aspekt.equalsIgnoreCase("generiert") || aspekt.equalsIgnoreCase("versendet") )
        getConnection().subscribeSender(this, getDavObjekt(), _datenBeschreibung, QUELLE);
      else
    	  getConnection().subscribeSender(this, getDavObjekt(), _datenBeschreibung, SENDER);
      //      System.out.println("Meldung-> zum Senden angemeldet fr " + getDavObjekt());
      _angemeldet = true;
    }

    catch (OneSubscriptionPerSendData e)
    {
      /* Es liegt bereits eine Sendeanmeldung von anderer Stelle vor
       * => hier nicht anmnelden
       */
    	if( false)
    	  System.out.println (" " + getDavObjekt().getPid() + 
            " ist bereits an anderer Stelle zum Senden angemeldet");
    }
  }
  
  /**
   * Gibt die mit dieser Instanz verbundenen Ressourcen wieder frei
   */
  public void dispose()
  {
    if (_angemeldet)
    {
      getConnection().unsubscribeSender(this, getDavObjekt(), _datenBeschreibung);
      _debug.config("Senden abgemeldet von " + _datenBeschreibung + " fr " + getDavObjekt());
      _angemeldet = false;
    }
  }
   
  /**
   * Unmittelbares Senden von Daten.
   * 
   * @param resultData die zu sendenden Daten
   * @return true, wenn das Senden erfolgreich war, false sonst
   */
  protected boolean send(ResultData resultData)
  {
    try
    {
      getConnection().sendData(resultData);
    }
    catch(SendSubscriptionNotConfirmed e)
    {
      if (SENDE_TIMEOUT > 0)
      {  
        // _debug.config("Warte max. " + SENDE_TIMEOUT + "ms auf Sendesteuerung fr " + getDavObjekt().getPid());
        System.out.println("Warte max. " + SENDE_TIMEOUT + "ms auf Sendesteuerung fr " + getDavObjekt().getPid());
        synchronized(this)
        {
          _warten = true;
          try
          {
            wait(SENDE_TIMEOUT);
          }
          catch(InterruptedException ex)
          {
            // tue nichts
          }
          _warten = false;
        }
        try
        {
          getConnection().sendData(resultData);
        }
        catch (SendSubscriptionNotConfirmed ex)
        {
          System.out.println("FEHLER: " + ex + 
              "\nKeine Sendesteuerung fr " + getDavObjekt().getPid() + " mit " + _datenBeschreibung);
          // _debug.warning("Keine Sendesteuerung fr " + getDavObjekt().getPid(), ex);
          return false;
        }
        return true;
      }
      else
      {
        System.out.println("FEHLER: " + e + 
            "\nKeine Sendesteuerung fr " + getDavObjekt().getPid() + " mit " + _datenBeschreibung);
        // _debug.warning("Keine Sendesteuerung fr " + getDavObjekt().getPid(), e);
        return false;
      }
    }
    catch (DataNotSubscribedException e)
    {
      System.out.println("FEHLER: " + e + 
          "\nKeine Sendeanmeldung bzw. Sendeanmeldung ungltig fr " + getDavObjekt().getPid() + " mit " + _datenBeschreibung);
      // _debug.error("Keine Sendeanmeldung bzw. Sendeanmeldung ungltig fr " + getDavObjekt().getPid(), e);
      return false;
    }
    return true;
  }
  
  /*
   * (Kein Javadoc)
   * @see de.bsvrz.dav.daf.main.ClientSenderInterface#dataRequest(de.bsvrz.dav.daf.main.config.SystemObject, de.bsvrz.dav.daf.main.DataDescription, byte)
   */
  public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
  {
    _debug.config("dataRequest() fr " + getDavObjekt().getPid() + " - Sender state: " + state);
    if (_warten && state == ClientSenderInterface.START_SENDING)
    {
      synchronized(this)
      {
        notify();
      }
    }
  }

  /*
   * (Kein Javadoc)
   * @see de.bsvrz.dav.daf.main.ClientSenderInterface#isRequestSupported(de.bsvrz.dav.daf.main.config.SystemObject, de.bsvrz.dav.daf.main.DataDescription)
   */
  public boolean isRequestSupported(SystemObject so, DataDescription dd)
  {
    return true;
  }
    
  /**
   * Aktuell unter dem Aspekt senden gesendete Daten 
   */
  private ResultData _datenSenden = null;
  
  /**
   * @return die unter dem Aspekt senden aktuell gesendeten Daten
   */
  public ResultData getDatenSenden()
  {
    return _datenSenden;
  }
  
  /**
   * setzt die unter dem Aspekt senden aktuell gesendeten Daten
   * 
   * @param datenSenden die aktuell gesendeten Daten
   */
  void setDatenSenden(ResultData datenSenden)
  {
    _datenSenden = datenSenden;
  }
  
  /**
   * Bildet eine sendefertige ResultData-Instanz fr das aktuelle Meldungsobjekt mit dem Zeitstempel der aktuellen Zeit
   * und einem DAV-Datensatz fr das bergebene Ereignis
   * 
   * @param ereignis das Ereignis, fr das der die Daten erzeugt werden sollen
   * 
   * @return ResultData Instanz 
   */
  public ResultData getResultData(String quelle, String zustand, String aspekt, String status)
  {
  	String text = null;
    Data data = getData(quelle, zustand, status);
    AttributeGroup atg = getConnection().getDataModel().getAttributeGroup("atg.rdsMeldung");
    
    if(aspekt.equalsIgnoreCase("generiert"))
      text = "asp.rdsGeneriert";
    
    if(aspekt.equalsIgnoreCase("bearbeitet"))
      text = "asp.rdsBearbeitet";

    if(aspekt.equalsIgnoreCase("versendet"))
      text = "asp.rdsVersendet";
    	
    Aspect asp = getConnection().getDataModel().getAspect(text);
    return new ResultData(getDavObjekt(), new DataDescription(atg, asp), getConnection().getTime(), data);
  }
    
  /**
   * Bildet einen DAV-Datensatz zur vorliegenden Meldung und initialisiert die relevanten Attribute bezg.
   * Location(s) und Ereignis
   * 
   * @param ereignis das Ereignis, fr das der Datensatz gebildet werden soll
   * 
   * @return den gebildeten DAV-Datensatz
   */
  private Data getData(String quelle, String zustand, String status)
  {
    AttributeGroup atg = getConnection().getDataModel().getAttributeGroup("atg.rdsMeldung");
    Data data = getConnection().createData(atg);
    
    data.getItem("InfoGUID").asTextValue().setText("");
    
    Data dataId = data.getItem("ID");
    dataId.getItem("IDOrganisation").asTextValue().setText("");
    dataId.getItem("IDNummer").asTextValue().setText("");
    
    Data dataVersion = data.getItem("Version");
    dataVersion.getItem("VersionGUID").asTextValue().setText("");
    
    Data dataVerwaltung = dataVersion.getItem("VerwaltungsInformationen");
    dataVerwaltung.getItem("AuthorisierendeOrganisationsEinheit").asTextValue().setText("");
    dataVerwaltung.getItem("AuthorisierenderNutzer").asTextValue().setText("");
    dataVerwaltung.getItem("AuthorisierungsZeit").asTimeValue().setMillis(1);
    dataVerwaltung.getItem("AuthorisierungsKommentar").asTextValue().setText("");    // dataVerwaltung.getItem("AuthorisierungsErgebnis").asUnscaledValue().set(0);
    dataVerwaltung.getItem("VersionsNummer").asUnscaledValue().set(1);
    
    dataVerwaltung.getItem("AktualisierungsZeit").asTimeValue().setMillis(getConnection().getTime());
    dataVerwaltung.getItem("AktivierungsZeit").asTimeValue().setMillis(1);
    dataVerwaltung.getItem("AblaufZeit").asTimeValue().setMillis(1);
    dataVerwaltung.getItem("ErinnerungsZeit").asTimeValue().setMillis(1);
    dataVerwaltung.getItem("ErinnerungsZeitOffset").asTimeValue().setMillis(0);
    dataVerwaltung.getItem("ErzeugungsZeit").asTimeValue().setMillis(getConnection().getTime());
    Data dataAenderungsInfo = dataVerwaltung.getItem("nderungsInformationen");
    dataAenderungsInfo.getItem("Veranlasser").asTextValue().setText("");
    dataAenderungsInfo.getItem("ModifikationsZeitpunkt").asTimeValue().setMillis(1);
    dataAenderungsInfo.getItem("ModifikationsKommentar").asTextValue().setText("");
    
    if(status.equalsIgnoreCase("0"))
      dataVerwaltung.getItem("Status").asTextValue().setText("Modifikation");

    if(status.equalsIgnoreCase("1"))
      dataVerwaltung.getItem("Status").asTextValue().setText("Aufhebung");

    if(status.equalsIgnoreCase("9"))
      dataVerwaltung.getItem("Status").asTextValue().setText("Lschung");
        
    dataVerwaltung.getItem("Prioritt").asUnscaledValue().set(0); // 0 == "Normal"
    dataVerwaltung.getItem("LandesKennung").asTextValue().setText("rdsLandesKennung.DE");

    if(zustand.equalsIgnoreCase("nicht quittiert"))
      dataVerwaltung.getItem("Zustand").asTextValue().setText("nicht quittiert");

    if(zustand.equalsIgnoreCase("quittiert"))
      dataVerwaltung.getItem("Zustand").asTextValue().setText("quittiert");

    if(zustand.equalsIgnoreCase("editiert"))
      dataVerwaltung.getItem("Zustand").asTextValue().setText("editiert");

    if(zustand.equalsIgnoreCase("verworfen"))
      dataVerwaltung.getItem("Zustand").asTextValue().setText("verworfen");

    if(quelle.equalsIgnoreCase("Automatik"))
      dataVerwaltung.getItem("Quelle").asTextValue().setText("automatisch");

    if(quelle.equalsIgnoreCase("Manuell"))
      dataVerwaltung.getItem("Quelle").asTextValue().setText("manuell");
    
    Data dataVerkehr = dataVersion.getItem("VerkehrsInformationen");
    dataVerkehr.getItem("RDSFormat").asUnscaledValue().set(0);
    
    Data dataNachrichten = dataVerkehr.getItem("Nachrichten");
    dataNachrichten.getItem("NachrichtenSprache").asUnscaledValue().set(1031);
    
    Data dataFreierText = dataVerkehr.getItem("FreierText");
    dataFreierText.getItem("Text").asTextValue().setText("");
    dataFreierText.getItem("TextSprache").asUnscaledValue().set(1031);
    
    Data dataLocationTabelleInfo = dataVerkehr.getItem("LocationTabelleInfo");
    dataLocationTabelleInfo.getItem("LocationTabelle").asUnscaledValue().set(1001);
    dataLocationTabelleInfo.getItem("LocationTabelleVersion").asTextValue().setText("");
    dataLocationTabelleInfo.getItem("LocationTabelleTyp").asUnscaledValue().set(0);
    dataLocationTabelleInfo.getItem("LocationTabelleName").asTextValue().setText("");
    
    Data dataLocationDaten = dataVerkehr.getItem("LocationDaten");
    Data dataRDSLocation = dataLocationDaten.getItem("RDSLocation"); // !!! Location Codes
     dataLocationDaten.getItem("TMCRichtung").asTextValue().setText("Positiv"); 
    dataLocationDaten.getItem("LocationPrimrEntfernung").asUnscaledValue().set(0);
    dataLocationDaten.getItem("LocationSekundrEntfernung").asUnscaledValue().set(0);
    dataLocationDaten.getItem("LocationVorLocation").asUnscaledValue().set(0);
    dataLocationDaten.getItem("LocationNachLocation").asUnscaledValue().set(0);
    
    Data dataEreignis = dataVerkehr.getItem("Ereignis");
    Data dataEreignisDaten = dataEreignis.getItem("EreignisDaten"); // !!! Ereignis Codes
    dataEreignis.getItem("EreignisTabelleNummer").asUnscaledValue().set(0);
    dataEreignis.getItem("EreignisTabelleVersion").asTextValue().setText("");
    NumberArray arrayEreignisTyp = dataEreignis.getUnscaledArray("EreignisTyp");
    arrayEreignisTyp.setLength(1);
    arrayEreignisTyp.getValue(0).set(0);
    
    Data dataZusatzAttribute = dataVerkehr.getItem("ZusatzAttribute");
    
    dataVerkehr.getItem("AlertCCode").asTextValue().setText("");
    
    return data;
  }
   
}


